 --------------------------------------------------------------------------------
-- Weapon Ice Spear
-- Havoc Weapon Pack
-- Script by Eric Killeen (v1.1)
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.icespear={}
cc.icespear.spear={}

-- Load & Prepare Ressources
cc.icespear.gfx_wpn=loadgfx("weapons/icespear.bmp")				-- Weapon Image
setmidhandle(cc.icespear.gfx_wpn)
cc.icespear.sfx_attack=loadsfx("arrow_shoot.ogg")				-- Attack Sound
cc.icespear.sfx_impact=loadsfx("arrow_impact.ogg")                              -- Impact Sound


--------------------------------------------------------------------------------
-- Weapon: Ice Spear
--------------------------------------------------------------------------------

cc.icespear.id=addweapon("cc.icespear","Ice Spear",cc.icespear.gfx_wpn,5)	-- Add Weapon (1 use)
cc.icespear.ammo=2												-- 6 Bullets

function cc.icespear.draw()										-- Draw
        -- Setup draw mode
        setblend(blend_light)
	setalpha(1)
	setcolor(166,166,255)
	setscale(1,1)
	if cc.icespear.ammo-weapon_shots>0 then
            drawinhand(cc.icespear.gfx_wpn,6,0)
	end
        -- HUD ammobar
	if cc.icespear.ammo-weapon_shots>0 then
		hudammobar(cc.icespear.ammo-weapon_shots,cc.icespear.ammo)
	end
	-- HUD Crosshair
	if cc.icespear.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.icespear.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Start burst
	if attack==1 then
		-- No more weapon switching!
		useweapon(0)
		weapon_mode=1
	end
	-- Attack
	if weapon_shots<cc.icespear.ammo and weapon_timer<=0 and weapon_mode==1 then
		-- Reset Timer
		weapon_timer=15
		-- Attack
		playsound(cc.icespear.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.icespear.spear.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		randomseed(weapon_shots*12345)
		offset = (weapon_shots - 0.5)* 2
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)+offset))*9.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)+offset))*9.0
		--
		projectiles[id].timer = 70;
		projectiles[id].impact = 0;
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*0.2
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*0.2
		for i=1,3,1 do
			if cc.icespear.spear.move(id)==1 then
				break
			end
		end
		projectiles[id].ignore=0
		-- Effects
		recoil(3)
		--particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		--particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		particle(p_bullet,getplayerx(0)+(getplayerdirection(0)*5),getplayery(0)+3)
		-- End Turn
		if (weapon_shots>=cc.icespear.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: spear
--------------------------------------------------------------------------------

cc.icespear.spear.id=addprojectile("cc.icespear.spear")			-- Add Projectile

function cc.icespear.spear.draw(id)							-- Draw
	local ii
        -- Setup draw mode
        setblend(blend_light)
	setalpha(1)
	setcolor(166,166,255)
	setscale(1,1)

	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	ii = cc.icespear.gfx_wpn
        drawimage(ii, projectiles[id].x,projectiles[id].y)
        -- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.icespear.spear.update(id)
	-- Wind + Gravity influence on speed
	if projectiles[id].impact ~= 0 then
                projectiles[id].timer=projectiles[id].timer-1
        end
	if projectiles[id].timer<=0 then
		playsound(cc.icegrenade.sfx_freeze)
		-- Crater
		blue=math.random(200,255)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
		else
			terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
		end
		-- Freeze Players
		players=playertable()
		for i=1,#players,1 do
			if getplayerhealth(players[i])>0 then
				if math.sqrt((getplayerx(players[i])-projectiles[id].x)^2+(getplayery(players[i])-projectiles[id].y)^2)<=45 then
					playerstate(players[i],state_frozen,1)
				end
			end
		end
		-- Particles
		particle(p_muzzle,projectiles[id].x,projectiles[id].y)
		particlecolor(255,255,255)
		for angle=0,350,10 do
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			particlespeed(math.sin(math.rad(angle))*2,-math.cos(math.rad(angle))*2)
			particlecolor(255,255,255)
		end
		-- Free projectile
		freeprojectile(id)
		return 1
	end
        if projectiles[id].impact > 0 then
	        -- handle player stickins
	        if getplayerhealth(projectiles[id].impact)>0 then
                   projectiles[id].x=getplayerx(projectiles[id].impact) - projectiles[id].edx
	           projectiles[id].y=getplayery(projectiles[id].impact) - projectiles[id].edy
	           if projectiles[id].timer == 60 then
                      playerstate(projectiles[id].impact,state_frozen,1)
                      -- Particles
		      particle(p_muzzle,projectiles[id].x,projectiles[id].y)
		      particlecolor(255,255,255)
		      for angle=0,350,10 do
		 	  particle(p_smoke,projectiles[id].x,projectiles[id].y)
		 	  particlespeed(math.sin(math.rad(angle))*2,-math.cos(math.rad(angle))*2)
			  particlecolor(255,255,255)
		      end
                   end
                else
                   projectiles[id].impact = 0
                end
	end
        if projectiles[id].impact == 0 then
	        -- flying
	        projectiles[id].sx=projectiles[id].sx+getwind()*0.1
	        projectiles[id].sy=projectiles[id].sy+getgravity()*1.5  -- bow moves 1.5
	        cc.icespear.spear.move(id)
	        -- Scroll to projectile
	        scroll(projectiles[id].x,projectiles[id].y)
	end

end

function cc.icespear.spear.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
                -- Collision
                if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*8,projectiles[id].y-math.cos(math.rad(rot))*8)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if projectiles[id].impact==0 then
                                   -- Effects
				   playsound(cc.icespear.sfx_impact)
				   particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*4,projectiles[id].y-math.cos(math.rad(rot))*4)
				   particlefadealpha(0.006)
                                end
                                -- Cause damage
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					if projectiles[id].impact ~= playercollision() then
					     projectiles[id].impact=playercollision();
                                             projectiles[id].edx = getplayerx(projectiles[id].impact) - projectiles[id].x
                                             projectiles[id].edy = getplayery(projectiles[id].impact) - projectiles[id].y
                                             playerpush(playercollision(),projectiles[id].sx*0.85,projectiles[id].sy*0.85)
					     playerdamage(playercollision(),24)
					     blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
					end
					if getplayerhealth(projectiles[id].impact)>0 then
                                             projectiles[id].sx = projectiles[id].sx * 0.05
                                             projectiles[id].sy = projectiles[id].sy * 0.05
                                        else
                                             projectiles[id].sx = projectiles[id].sx * 0.85
                                             projectiles[id].sy = projectiles[id].sy * 0.85
                                             projectiles[id].impact = 0
                                        end
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),20)
					projectiles[id].impact=-1;
                                else
                                        projectiles[id].impact=-1;
				end

                                return 1
			end
		else
			projectiles[id].impact=0
		end



		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end