 --------------------------------------------------------------------------------
-- Weapon Stake of Van Helsing
-- Havoc Weapon Pack
-- Script by Eric Killeen (v1.0)
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.vanhelsing={}
cc.vanhelsing.stake={}

-- Load & Prepare Ressources
cc.vanhelsing.gfx_wpn=loadgfx("weapons/vanhelsing.bmp")                 -- Weapon Image
setmidhandle(cc.vanhelsing.gfx_wpn)
cc.vanhelsing.sfx_attack=loadsfx("throw.ogg")                           -- Attack Sound
cc.vanhelsing.sfx_impact=loadsfx("arrow_impact.ogg")                    -- Impact Sound
cc.vanhelsing.sfx_leach=loadsfx("firehit.ogg")                          -- Health Leach Sound

--------------------------------------------------------------------------------
-- Weapon: Stake of Van Helsing
--------------------------------------------------------------------------------

cc.vanhelsing.id=addweapon("cc.vanhelsing","Van Helsing",cc.vanhelsing.gfx_wpn,5)       -- Add Weapon (1 use)
cc.vanhelsing.ammo=1                                                                                            -- 6 Bullets

function cc.vanhelsing.draw()                                                                           -- Draw
        -- Setup draw mode
        setblend(blend_alpha)
        setalpha(1)
        setcolor(166,166,255)
        setscale(1,1)
        if cc.vanhelsing.ammo-weapon_shots>0 then
           drawinhand(cc.vanhelsing.gfx_wpn,6,0)
        end
        -- HUD ammobar
        if cc.vanhelsing.ammo-weapon_shots>0 then
                hudammobar(cc.vanhelsing.ammo-weapon_shots,cc.vanhelsing.ammo)
        end
        -- HUD Crosshair
        if cc.vanhelsing.ammo-weapon_shots>0 then
                hudcrosshair(7,3)
        end
end

function cc.vanhelsing.attack(attack)                                                           -- Attack
        -- Decrement timer
        if weapon_timer>0 then
                weapon_timer=weapon_timer-1
        end
        -- Start burst
        if attack==1 then
                -- No more weapon switching!
                useweapon(0)
                weapon_mode=1
        end
        -- Attack
        if weapon_shots<cc.vanhelsing.ammo and weapon_timer<=0 and attack==1 then
                -- Reset Timer
                weapon_timer=15
                -- Attack
                playsound(cc.vanhelsing.sfx_attack)
                weapon_shots=weapon_shots+1
                id=createprojectile(cc.vanhelsing.stake.id)
                projectiles[id]={}
                -- Ignore collision with current player at beginning
                projectiles[id].ignore=playercurrent()
                -- Set initial position of projectile
                projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
                projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
                -- Set speed of projectile
                randomseed(weapon_shots*12345)
                offset = (weapon_shots - 0.5)* 2
                projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)+offset))*9.0
                projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)+offset))*9.0
                --
                projectiles[id].timer = 200;
                projectiles[id].vampire = 0;
                projectiles[id].impact = 0;
                projectiles[id].spin = 0;
                -- Initial movement
                projectiles[id].x=projectiles[id].x-projectiles[id].sx*0.2
                projectiles[id].y=projectiles[id].y-projectiles[id].sy*0.2
                for i=1,3,1 do
                        if cc.vanhelsing.stake.move(id)==1 then
                                break
                        end
                end
                projectiles[id].ignore=0
                -- Effects
                recoil(3)
                --particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
                --particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
                particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
                particlefadealpha(0.005)
                particle(p_bullet,getplayerx(0)+(getplayerdirection(0)*5),getplayery(0)+3)
                -- End Turn
                if (weapon_shots>=cc.vanhelsing.ammo) then
                        endturn()
                end
        end
end

--------------------------------------------------------------------------------
-- Projectile: stake
--------------------------------------------------------------------------------

cc.vanhelsing.stake.id=addprojectile("cc.vanhelsing.stake")                     -- Add Projectile

function cc.vanhelsing.stake.draw(id)                                                   -- Draw
        local ii
        -- Setup draw mode
        setblend(blend_alpha)
        setalpha(1)
        setcolor(255,255,255)
        setscale(1,1)

        -- Calculate projectile rotation
        if projectiles[id].impact == 0 then
           setrotation(projectiles[id].spin + math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
        else
           setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
        end
        -- Draw projectile
        ii = cc.vanhelsing.gfx_wpn
        drawimage(ii, projectiles[id].x,projectiles[id].y)
        -- Draw Arrow if out of Screen
        outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.vanhelsing.stake.update(id)
        local vhp
        -- Wind + Gravity influence on speed
        if projectiles[id].impact ~= 0 then
                projectiles[id].timer=projectiles[id].timer-1
        end
        if projectiles[id].timer<=0 then
                -- Particles
                particle(p_muzzle,projectiles[id].x,projectiles[id].y)
                particlecolor(255,55,55)
                for angle=0,340,20 do
                        particle(p_smoke,projectiles[id].x,projectiles[id].y)
                        particlespeed(math.sin(math.rad(angle))*1.2,-math.cos(math.rad(angle))*1.2)
                        particlecolor(255,55,55)
                end
                -- Free projectile
                freeprojectile(id)
                return 1
        end
        if projectiles[id].impact > 0 then
                -- handle player stickins
                if getplayerhealth(projectiles[id].impact)>1 then
                   projectiles[id].x=getplayerx(projectiles[id].impact) - projectiles[id].edx
                   projectiles[id].y=getplayery(projectiles[id].impact) - projectiles[id].edy
                   if projectiles[id].vampire < 60 then
                      projectiles[id].vampire = projectiles[id].vampire + 1
                   else
                      playsound(cc.vanhelsing.sfx_leach)
                      projectiles[id].vampire = 0
                      local x, y

                      -- Transfer Health
                      vhp = math.min(getplayerhealth(projectiles[id].impact) - 1, 10)
                      playerheal(0, 2 * vhp / 3)
                      playerdamage(projectiles[id].impact,vhp)
                      -- Target Particles
                      x=getplayerx(projectiles[id].impact)
                      y=getplayery(projectiles[id].impact)+3
                      particle(p_muzzle,x,y)
                      particlesize(1,1)
                      particlecolor(255,0,0)
                      particlealpha(1.0)
                      particlefadealpha(0.01)
                      particle(p_muzzle,x,y)
                      particlesize(1,2)
                      particlecolor(255,0,0)
                      particlealpha(1.0)
                      particlefadealpha(0.02)
                      particlerotation(0)
                      for j=1,15,1 do
                                particle(p_flare,x-10.0+math.random()*20.0,y+math.random()*10.0)
                                particlesize(3.0,3.0)
                                particlecolor(255,0,0)
                                particlespeed(0,math.random(2,15)*-0.1)
                                particlefadealpha(0.05)
                      end
                      -- Me Particles
                      x=getplayerx(0)
                      y=getplayery(0)+3
                      particle(p_muzzle,x,y)
                      particlesize(1,1)
                      particlecolor(255,0,0)
                      particlealpha(1.0)
                      particlefadealpha(0.01)
                      particle(p_muzzle,x,y)
                      particlesize(1,2)
                      particlecolor(255,0,0)
                      particlealpha(1.0)
                      particlefadealpha(0.02)
                      particlerotation(0)
                      for j=1,15,1 do
                                particle(p_flare,x-10.0+math.random()*20.0,y+math.random()*10.0)
                                particlesize(3.0,3.0)
                                particlecolor(200,255,200)
                                particlespeed(0,math.random(2,15)*-0.1)
                                particlefadealpha(0.01)
                      end
                   end
                end
        end
        if projectiles[id].impact == 0 then
                -- flying
                -- projectiles[id].spin = projectiles[id].spin + 1.2
                projectiles[id].sx=projectiles[id].sx+getwind()*0.1
                projectiles[id].sy=projectiles[id].sy+getgravity()*1.5  -- bow moves 1.5
                particle(p_flare,projectiles[id].x-10.0+math.random()*20.0,projectiles[id].y+math.random()*10.0)
                particlesize(3.0,3.0)
                particlecolor(200,50,20)
                particlespeed(0,math.random(2,15)*-0.1)
                particlefadealpha(0.04)
                cc.vanhelsing.stake.move(id)
                -- Scroll to projectile
                scroll(projectiles[id].x,projectiles[id].y)
        end

end

function cc.vanhelsing.stake.move(id)
        rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
        projectiles[id].spin = projectiles[id].spin + 14.2
        -- Move (in substep loop for optimal collision precision)
        msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
        msubx=projectiles[id].sx/msubt
        msuby=projectiles[id].sy/msubt
        for i=1,msubt,1 do
                projectiles[id].x=projectiles[id].x+msubx
                projectiles[id].y=projectiles[id].y+msuby
                -- Collision
                if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*8,projectiles[id].y-math.cos(math.rad(rot))*8)==1 then
                        if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
                                if projectiles[id].impact==0 then
                                   -- Effects
                                   playsound(cc.vanhelsing.sfx_impact)
                                   particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*4,projectiles[id].y-math.cos(math.rad(rot))*4)
                                   particlefadealpha(0.006)
                                end
                                -- Cause damage
                                if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
                                        if projectiles[id].impact ~= playercollision() then
                                             projectiles[id].impact=playercollision();
                                             projectiles[id].edx = getplayerx(projectiles[id].impact) - projectiles[id].x
                                             projectiles[id].edy = getplayery(projectiles[id].impact) - projectiles[id].y
                                             playerpush(playercollision(),projectiles[id].sx*0.05,projectiles[id].sy*0.05)
                                             playerdamage(playercollision(),4)
                                             blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)

                                        end
                                        if getplayerhealth(projectiles[id].impact)>0 then
                                             projectiles[id].sx = projectiles[id].sx * 0.05
                                             projectiles[id].sy = projectiles[id].sy * 0.05
                                        else
                                             projectiles[id].impact = 0
                                        end
                                elseif objectcollision()>0 then
                                        objectdamage(objectcollision(),20)
                                        projectiles[id].impact=-1;
                                else
                                        projectiles[id].impact=-1;
                                end

                                return 1
                        end
                else
                        projectiles[id].impact=0
                end
                -- Water
                if (projectiles[id].y)>getwatery()+5 then
                        -- Effects
                        particle(p_waterhit,projectiles[id].x,projectiles[id].y)
                        if math.random(1,2)==1 then
                                playsound(sfx_hitwater2)
                        else
                                playsound(sfx_hitwater3)
                        end
                        -- Free projectile
                        freeprojectile(id)
                        return 1
                end
        end
end
